var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; };

function _objectWithoutProperties(obj, keys) { var target = {}; for (var i in obj) { if (keys.indexOf(i) >= 0) continue; if (!Object.prototype.hasOwnProperty.call(obj, i)) continue; target[i] = obj[i]; } return target; }

import React from "react";
import graphql from "graphql-anywhere";
import glamorous from "glamorous";
import { Broadcast, Subscriber } from "react-broadcast";
import PropTypes from "prop-types";

var smoosh = function smoosh(object) {
    return Object.assign.apply(Object, [{}].concat(function _flatten(objectBit) {
        var _ref;

        return (_ref = []).concat.apply(_ref, Object.keys(objectBit).map(function (key) {
            var _ref2;

            return _typeof(objectBit[key]) === "object" ? _flatten(objectBit[key]) : (_ref2 = {}, _ref2[key] = objectBit[key], _ref2);
        }));
    }(object)));
};

var resolver = function resolver(fieldName, root, args, context, _ref3) {
    var resultKey = _ref3.resultKey;

    // if it's an aliased query add alias as prop
    if (fieldName !== resultKey) {
        return _extends({}, root[fieldName], args, {
            prop: resultKey
        });
    }

    var res = root[fieldName];
    var rootUnit = root && root.unit;
    var argsUnit = args && args.unit;
    var argsVariant = args && args.variant;

    if (argsUnit || rootUnit) {
        var unit = argsUnit || rootUnit;
        res = root[fieldName] + unit;
    }

    if (argsVariant) {
        res = root[fieldName][argsVariant];
    }

    // if has prop then use it as the key
    if (root.prop) {
        var _ref4;

        return _ref4 = {}, _ref4[root.prop] = res, _ref4;
    }

    return res;
};

// Main function to generate glamorous component with styles
var gqlCSS = function gqlCSS(styles) {
    return function (query) {
        var component = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : "div";
        var variables = arguments[2];

        var generatedStyles = graphql(resolver, query, styles, null, variables);
        var smooshedStyles = smoosh(generatedStyles);

        // Returns just smooshed styles if component is false
        if (component === false) {
            return smooshedStyles;
        }

        // Create glamorous component
        return glamorous(component || "div")(smooshedStyles);
    };
};

// Also export component for more declarative API
var GqlCSS = function GqlCSS(_ref5) {
    var styles = _ref5.styles,
        query = _ref5.query,
        _ref5$component = _ref5.component,
        component = _ref5$component === undefined ? "div" : _ref5$component,
        variables = _ref5.variables,
        rest = _objectWithoutProperties(_ref5, ["styles", "query", "component", "variables"]);

    return React.createElement(
        Subscriber,
        { quiet: true, channel: "graphqlcss" },
        function (contextStyles) {
            var Component = gqlCSS(styles || contextStyles || "")(query, component, variables);
            return React.createElement(Component, rest);
        }
    );
};
export { GqlCSS };
GqlCSS.propTypes = {
    styles: PropTypes.object,
    query: PropTypes.object,
    variables: PropTypes.object,
    component: PropTypes.oneOfType([PropTypes.string, PropTypes.node])
};

// Export provider to broadcast styles to child GqlCSS components
export var GqlCSSProvider = function GqlCSSProvider(_ref6) {
    var styles = _ref6.styles,
        children = _ref6.children;
    return React.createElement(
        Broadcast,
        { channel: "graphqlcss", value: styles },
        React.createElement(
            "span",
            null,
            children
        )
    );
};
GqlCSSProvider.propTypes = {
    styles: PropTypes.object,
    children: PropTypes.node
};

// Export HOC - uses render props underneath, because it's awesome
export var withGqlCSS = function withGqlCSS(styles, query, variables) {
    return function (Component) {
        return function (props) {
            return React.createElement(
                WithGqlCSS,
                { styles: styles, query: query, variables: variables },
                function (_ref7) {
                    var gqlStyles = _ref7.gqlStyles;
                    return React.createElement(Component, _extends({}, props, { gqlStyles: gqlStyles }));
                }
            );
        };
    };
};

// Export render props component
var WithGqlCSS = function WithGqlCSS(_ref8) {
    var styles = _ref8.styles,
        query = _ref8.query,
        variables = _ref8.variables,
        children = _ref8.children,
        rest = _objectWithoutProperties(_ref8, ["styles", "query", "variables", "children"]);

    var processedStyles = gqlCSS(styles)(query, false, variables);
    return children(_extends({ gqlStyles: processedStyles }, rest));
};

export { WithGqlCSS };
export default gqlCSS;